@@ -41,6 +41,17 @@ int* qqwing_generate_puzzle(int difficulty)
     const int BOARD_SIZE = 81;
     SudokuBoard board;
 
+    // Seed RNG exactly once.
+    static GMutex mutex;
+    static bool seeded = false;
+    g_mutex_lock(&mutex);
+    if (!seeded)
+    {
+        srand(time(NULL));
+        seeded = true;
+    }
+    g_mutex_unlock(&mutex);
+
     board.setRecordHistory(true);
     board.setLogHistory(false);
     board.setPrintStyle(SudokuBoard::ONE_LINE);
@@ -63,6 +74,20 @@ int* qqwing_generate_puzzle(int difficulty)
     return copy;
 }
 
+/*
+ * Count the number of solutions of a puzzle
+ * but return 2 if there are multiple.
+ * Returns 0 if the puzzle is not valid.
+ */
+int qqwing_count_solutions_limited(int* puzzle)
+{
+    SudokuBoard board;
+    if (!board.setPuzzle(puzzle))
+        return 0;
+
+    return board.countSolutionsLimited();
+}
+
 /*
  * Print the stats gathered while solving the puzzle given as input.
  */
